import npyscreen, time, curses
import database, myFFT

global db 

# TODO: Rewrite the whole thing ... I hate GUI

tableView, detailView = [], []
for ii in database.dbElem:
    if ii[4]: tableView+=[ii]
    if ii[5]: detailView+=[ii]

serverPort = None

def updateTable(grid):
    global serverPort,db
    db = database.clientSend(serverPort,['DB'])
    data = []
    for ii in db:
        rec = db[ii]
        if rec['prio']>0:
            data = data + [[rec['freq']]+[jj[3]%rec[jj[0]] for jj in tableView]]
    data.sort()
    grid.values = [ii[1:] for ii in data] 

class SDRgui(npyscreen.NPSAppManaged):
    keypress_timeout_default = 50
    def onStart(self):
        self.SDRForm = self.addForm("MAIN",MainForm,name="SDRpeek")
        self.DetailsForm = self.addForm("Details",DetailsForm,name="Details")
        self.HelpForm = self.addForm("Help",HelpForm,name="Help")
        
    def change_form(self, name):
        self.switchForm(name)
        self.resetHistory()

class myGrid(npyscreen.SimpleGrid):
    def custom_print_cell(self, actual_cell, cell_display_value):
        if cell_display_value =="3  ":
            actual_cell.color = 'DANGER'
        elif cell_display_value =="0  ":
            actual_cell.color = 'GOOD'
        else: 
            actual_cell.color = 'DEFAULT'

class DetailsForm(npyscreen.FormBaseNew):
    def create(self):
        self.myTitle = self.add(npyscreen.FixedText,value= "NA")
        #self.myDate = self.add(npyscreen.FixedText,value= "NA")
        self.grid = self.add(myGrid,value= [["NA"]])
        self.add_handlers({" ": self.change_forms})
        self.add_handlers({"f": self.fft})
        self.add_handlers({"1": self.gqrx})

    def gqrx(self,*args):
        chan = self.parentApp.SDRForm.grid.edit_cell
        rFreq = self.parentApp.SDRForm.grid.values[chan[0]][0]
        rec = db[int(float(rFreq)*1e6)]
        if self.grid.edit_cell!=None:
            indx = self.grid.edit_cell[0]
            myFFT.startGQRX([rec['looktable'][indx]])

    def fft(self,*args):
        chan = self.parentApp.SDRForm.grid.edit_cell
        rFreq = self.parentApp.SDRForm.grid.values[chan[0]][0]
        rec = db[int(float(rFreq)*1e6)]
        if self.grid.edit_cell!=None:
            indx = self.grid.edit_cell[0]
            myFFT.startFFT([rec['looktable'][indx]])


    def updateText(self):
        global db
        chan = self.parentApp.SDRForm.grid.edit_cell
        rFreq = self.parentApp.SDRForm.grid.values[chan[0]][0]
        rec = db[int(float(rFreq)*1e6)]
        date = time.strftime('%y%m%d: %H:%M',time.localtime(rec['time']))
        self.myTitle.value = 'Freq: %4.4f SNR %2.0f Last peeked %s'%(rec['freq'], rec['snr'],date)
        #self.myTitle.value = rec['looktable']
        savedSignals = []
        for ii in rec['looktable']:
            date = time.strftime('%y%m%d: %H:%M:%S',time.localtime(ii[2]))
            savedSignals += [['%4.3f'%(ii[0]/1e6) ,'%4.3f'%(ii[1]/1e6), '%s'%date, '%s'%ii[3]]]
        self.grid.values = savedSignals

    def change_forms(self, *args, **keywords):
        if self.name == "SDRpeek":
            change_to = "Details"
        else:
            change_to = "MAIN"

        # Tell the MyTestApp object to change forms.
        self.parentApp.change_form(change_to)

class HelpForm(npyscreen.ActionForm):
    def create(self):
        txt = """
        i      : ignore freq
        f      : plot fft
        [space]: more details
        m      : record more instances of signal
        1      : show last signal in gqrx
        g      : TODO GNU radio
        """
        self.myTitle = self.add(npyscreen.TitlePager,name='Help',
                                values=txt.split('\n'))
    def on_ok(self, *args, **keywords):
        self.parentApp.change_form('MAIN')
    def on_cancel(self, *args, **keywords):
        self.parentApp.change_form('MAIN')

class MainForm(npyscreen.FormBaseNew):
    def create(self):
        str = ''.join(['%-11s'%ii[1] for ii in tableView])
        tmp = self.add(npyscreen.TitleFixedText, name = 'Press ? for help', value= str,begin_entry_at=0)
        self.grid = self.add(myGrid)
        self.grid.column_width_requested = 10
        updateTable(self.grid)
        self.add_handlers({"i": self.ignore})
        self.add_handlers({"f": self.fft})
        self.add_handlers({"m": self.moreLook})
        self.add_handlers({" ": self.details})
        self.add_handlers({"?": self.helpMessage})
        self.keypress_timeout_default = 1
        self.add_handlers({"1": self.gqrx})


    def while_waiting(self):
        updateTable(self.grid)
        self.grid.display()

    def details(self,*args):
        if self.grid.edit_cell!=None:
            self.parentApp.change_form("Details")
            self.parentApp.DetailsForm.updateText()

    def ignore(self,*args):
        if self.grid.edit_cell!=None:
            rFreq = self.grid.values[self.grid.edit_cell[0]][0]
            database.clientSend(serverPort,['Ignore',int(float(rFreq)*1e6)])
            updateTable(self.grid)
            self.grid.display()

    def moreLook(self,*args):
        if self.grid.edit_cell!=None:
            rFreq = self.grid.values[self.grid.edit_cell[0]][0]
            database.clientSend(serverPort,['MoreLook',int(float(rFreq)*1e6)])
            updateTable(self.grid)
            self.grid.display()

    def fft(self,*args):
        if self.grid.edit_cell!=None:
            rFreq = self.grid.values[self.grid.edit_cell[0]][0]
            rec = db[int(float(rFreq)*1e6)]
            myFFT.startFFT(rec['looktable'])

    def gqrx(self,*args):
        if self.grid.edit_cell!=None:
            rFreq = self.grid.values[self.grid.edit_cell[0]][0]
            rec = db[int(float(rFreq)*1e6)]
            myFFT.startGQRX([rec['looktable'][-1]])

    def helpMessage(self,*args):
        self.parentApp.change_form("Help")

    def change_forms(self, *args, **keywords):
        if self.name == "SDRpeek":
            change_to = "Details"
        else:
            change_to = "MAIN"

        # Tell the MyTestApp object to change forms.
        self.parentApp.change_form(change_to)
        
def main(SP):
    global serverPort
    serverPort = SP
    print "hej",SP

    app = SDRgui()
    app.run()

if __name__ == '__main__':
    main()
