"""
Simple "database" using sockets to communicate
The server class starts a new thread listening unless there is a 
file "database_server" containing hostname:port

Thought of using mysql but ... too lazy to learn 
"""

# TODO: Scan range option .... now it's hardcoded into the event ...

import socket, os, string, pickle, glob, time, sys
import numpy as N

""" Database description ['dict name','Display name',default,'display format',Show in table, show in details]"""
dbElem = [["freq","Freq(MHz)",None,"%4.3f",False,True],
          ["rfreq","Freq(MHz)",None,"%4.1f",True,False],
          ["snr","SNR",None,'%2.0f',True,True],
          ["fwhm","FWHM(kHz)",None,'%4.0f',True,True],
          ["peekcnt","#Peek",1,'%i',True,True],
          ["lookcnt","#Saved",0,'%i',False,False],
          ["looktries","#Tries",0,'%i',True,False],
          ["lookmax","NA",200,'%i',False,False],
          ["prio","Prio",5,'%i',False,True],
          ["looknr","#Want",3,'%i  ',True,True],
          ["time","Time",None,None,False,False],
          ["looktable","NA",[],"NA",False,False]]

def clientSend(serverPort,data):
    # Send and then recieve package
    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    sock.connect(serverPort)
    sock.sendall(pickle.dumps(data))
    sock.shutdown(socket.SHUT_WR)
    reply = recieve(sock,serverPort)
    sock.close()
    return reply 

def recieve(sock,serverPort):
    mesg, new = '', sock.recv(serverPort[1])
    while len(new)>0:
        mesg += new 
        new = sock.recv(serverPort[1])
    return pickle.loads(mesg)

def StartDB(port,verbose=False):
    from multiprocessing import Process
    
    db = Process(target=procRunDB,args=(port,verbose))
    db.start()
    

def procRunDB(port,verbose):
    with server(port,verbose=verbose) as db:
        while True:
            db.database()

class server():
    def __init__(self,port,start=True,verbose=False):
        self.verbose = verbose
        self.remoteDB = os.path.isfile('database_server')
        if (not start) and (not self.remoteDB):
            sys.exit('Cannot find database')
        if start and self.remoteDB: 
            sys.exit('File: database_server indicate that DB already running')
        if start:
            self.serverSock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            self.serverSock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
            self.serverSock.bind(('',port))
            self.serverSock.listen(5)
            if verbose: print "Created database listening on port %i"%port

            with open('database_server','w') as f:
                f.write(socket.gethostname()+":%i\n"%port)
            self.readDatabase()
            self.serverPort = (socket.gethostname(),port)
        if self.remoteDB:
            with open('database_server','r') as f:
                data = string.split(f.readline(),':')
            self.serverPort = (data[0], int(data[1]))
            if verbose: print "Will try to use database %s:%s"%(data[0],data[1])
    
    def database(self):
        # Listen for commands and write database every 5 min
        if self.verbose: print "Listening",self.serverSock
        (clientSock, address) = self.serverSock.accept()
        if self.verbose: print "Conection from ",address
        mesg = recieve(clientSock,self.serverPort)
        if self.verbose: print "Recv from ",address
        reply = self.event(mesg)
        clientSock.sendall(pickle.dumps(reply))
        if self.verbose: print "Sent to ",address
        clientSock.close()
        if time.time()-self.dbTime>300:            
            self.flush_database()
            self.dbTime=time.time()

    def event(self,mesg):
        db = self.db
        if mesg[0]=="Peek?":
            # Return list of frequencies to scan
            return [80e6+ii*2e6+(N.random.rand()-0.5)*1e6 for ii in range(300)]
        if mesg[0]=='PeekResults':
            for ii in mesg[1]:
                rFreq = int(ii[0]*1e6/100.0e3+0.5)*100000
                if rFreq in db:
                    db[rFreq]['peekcnt']+=1
                else:
                    db[rFreq]={jj[0]:jj[2] for jj in dbElem}
                    db[rFreq]['looktable'] = [] # Otherwise points to same list ...
                rec = db[rFreq]
                rec['fwhm'], rec['snr'] = ii[1]*1e3, ii[2]
                rec['time'], rec['freq'] = time.time(), ii[0]
                rec['rfreq'] = rFreq/1e6
            return True
        if mesg[0]=='DB':
            return self.db
        if mesg[0]=='Look?':
            lfreq = []
            for ii,rec in db.iteritems():
                if rec['looknr']>0 and rec['looktries']<rec['lookmax']:
                    lfreq+=[rec['freq']*1e6]
                    rec['looktries']+=1
            return lfreq
        if mesg[0]=='LookResults':
            lFreq,Freq,fn = mesg[1]
            rFreq = int(lFreq/100.0e3+0.5)*100000
            rec=db[rFreq]
            rec['looknr']-=1
            rec['lookcnt']+=1
            rec['looktable']+=[[lFreq,Freq,time.time(),fn]]
        if mesg[0]=='Ignore':
            rec = db[mesg[1]]
            rec['looknr'], rec['prio'] = 0,0
        if mesg[0]=='MoreLook':
            rec = db[mesg[1]]
            rec['looknr'] += 5
            rec['lookmax'] += 200

    def readDatabase(self):
        # Read existing database or create new
        fns, done = glob.glob('*.mydb'), False
        fns.sort(reverse=True)
        for fn in fns:
            try:
                with open(fn,'r') as f:
                    self.db = pickle.load(f)
                    if self.verbose: print "Read db:%s"%fn
                break
            except: pass
        else:
            if self.verbose: print "Starting with empty db"
            self.db={} # New empty database
        self.dbTime = time.time()


    def flush_database(self):
        # Pickle database to file keeping the last 5 files
        if self.verbose: print "DB flushed"
        with open(time.strftime('%y%m%d%H%M%S.mydb'),'w') as f:
            pickle.dump(self.db,f)
        fns = glob.glob('*.mydb')
        fns.sort(reverse=True)
        for fn in fns[5:]: 
            if self.verbose: print "Removed %s"%fn
            os.remove(fn)

    def __enter__(self):
        return self

    def __exit__(self,type,value,traceback):
        import traceback as TB
        TB.print_exception(type, value, traceback)
        TB.print_tb(traceback)
        # Try to achieve a clean exit ...
        if not self.remoteDB:
            self.flush_database()
            os.remove('database_server')
            if self.verbose: print 'Removed "database_server"'
        return False            

if __name__ == '__main__':
    
    if not  os.path.isfile('database_server'):
        with server(port=45345,verbose=True) as db:
            #db=server(port=45345,verbose=True,start=True)
            if not db.remoteDB:
                while True: db.database()
    else:
        db = server(port=45345,start=False)
        print "Client"
        print clientSend(db.serverPort,'hej')
        
